;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File:         exec-unix.el
; RCS:          $Header: $
; Description:  Execute a command, wait for it to finish, and return the exit
;		status.
; Author:       Darryl Okahata
; Created:      Thu Aug 10 15:44:08 1989
; Modified:     Sat Feb 23 17:11:49 1991 (Darryl Okahata)
; Language:     Lisp Interaction
; Package:      N/A
;
; (C) Copyright 1989, 1990, 1991, Hewlett-Packard, all rights reserved.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun execute-unix-command (buffer program &rest args)
  "Execute a unix command, wait for it for finish, and return the exit status.
This function creates a process, using BUFFER for output, and waits
for the process to exit.  The name of the program to run is given by
PROGRAM, and any remaining arguments, ARGS, are passed as command line
arguments to the program.  All arguments are strings.

Note that there are no provisions for giving input to the program.  If
you must give input to the program, create a file containing the input
for the process and pass the name of this file to a shell script that
starts up the program with standard input redirected to the given
file.

If BUFFER is nil, any output from the program will be throw away.

If a ^G is pressed while the program is running, the program will be
killed via a SIGINT, and this function will return nil.  All other
keypresses will be flushed.

Upon normal completion of the program, the numeric exit status of the
program will be returned.  Note that this function returns either nil
or a numeric value.
"
  (let (prg (process nil) status (return-status nil))
    (catch 'exit-process
      (progn
	(setq prg (list 'start-process "unix-process" buffer program))
	(if args
	  (if (listp args)
	    (setq prg (append prg args))
	    (setq prg (append prg (list args)))
	  )
	)
	(setq process (eval prg))
	(if (processp process)
	  (progn
	    (process-send-eof process)
	    (while (or (equal (setq status (process-status process))
			      'run)
		       (equal status 'stop))
	      (sit-for 1)
	      (if (input-pending-p)
		(if (equal (read-char) ?\^G)
		  (progn
		    (interrupt-process process t)
		    (throw 'exit-process nil)
		  )
		)
	      )
	    )
	    (setq return-status (process-exit-status process))
	  )
	)
      )
    )
    return-status
  )
)
